@@ -48,7 +48,11 @@ static PyObject *point_new(TSPoint point) {
     Py_XDECREF(column);
     return NULL;
   }
-  return PyTuple_Pack(2, row, column);
+
+  PyObject *obj = PyTuple_Pack(2, row, column);
+  Py_XDECREF(row);
+  Py_XDECREF(column);
+  return obj;
 }
 
 // Node
@@ -192,14 +196,12 @@ static PyObject *node_get_children(Node *self, void *payload) {
 static PyObject *node_get_child_count(Node *self, void *payload) {
   long length = (long)ts_node_child_count(self->node);
   PyObject *result = PyLong_FromLong(length);
-  Py_INCREF(result);
   return result;
 }
 
 static PyObject *node_get_named_child_count(Node *self, void *payload) {
   long length = (long)ts_node_named_child_count(self->node);
   PyObject *result = PyLong_FromLong(length);
-  Py_INCREF(result);
   return result;
 }
 
@@ -609,7 +611,7 @@ static PyObject *parser_set_language(Parser *self, PyObject *arg) {
     return NULL;
   }
 
-  TSLanguage *language = (TSLanguage *)PyLong_AsLong(language_id);
+  TSLanguage *language = (TSLanguage *)PyLong_AsVoidPtr(language_id);
   if (!language) {
     PyErr_SetString(PyExc_ValueError, "Language ID must not be null");
     return NULL;
@@ -708,6 +710,8 @@ static PyObject *query_captures(Query *self, PyObject *args, PyObject *kwargs) {
     PyObject *capture_node = node_new_internal(capture->node, node->tree);
     PyObject *capture_name = PyList_GetItem(self->capture_names, capture->index);
     PyList_Append(result, PyTuple_Pack(2, capture_node, capture_name));
+    Py_XDECREF(capture_node);
+    Py_XDECREF(capture_name);
   }
 
   return result;
@@ -803,12 +807,15 @@ static PyObject *query_new_internal(
 
 static PyObject *language_field_id_for_name(PyObject *self, PyObject *args) {
   TSLanguage *language;
+  PyObject *language_id;
   char *field_name;
   Py_ssize_t length;
-  if (!PyArg_ParseTuple(args, "ls#", &language, &field_name, &length)) {
+  if (!PyArg_ParseTuple(args, "Os#", &language_id, &field_name, &length)) {
     return NULL;
   }
 
+  language = (TSLanguage *)PyLong_AsVoidPtr(language_id);
+
   TSFieldId field_id = ts_language_field_id_for_name(language, field_name, length);
   if (field_id == 0) {
     Py_RETURN_NONE;
@@ -819,12 +826,15 @@ static PyObject *language_field_id_for_name(PyObject *self, PyObject *args) {
 
 static PyObject *language_query(PyObject *self, PyObject *args) {
   TSLanguage *language;
+  PyObject *language_id;
   char *source;
   Py_ssize_t length;
-  if (!PyArg_ParseTuple(args, "ls#", &language, &source, &length)) {
+  if (!PyArg_ParseTuple(args, "Os#", &language_id, &source, &length)) {
     return NULL;
   }
 
+  language = (TSLanguage *)PyLong_AsVoidPtr(language_id);
+
   return query_new_internal(language, source, length);
 }
 
